package server

import (
	"encoding/json"
	"game/common/model/user"
	"game/common/proto/pb"
	"game/common/rpcName"
	"game/common/utils"
	"github.com/fox/fox/ipb"
	"github.com/fox/fox/ksync"
	"github.com/fox/fox/log"
	"github.com/fox/fox/service"
)

func (s *MatchService) getGameUser(uid int64) (*user.GameUser, pb.ErrCode) {
	node, err := s.bindService.HashServiceNode(pb.ServiceTypeId_STI_DB, uid)
	if err != nil {
		log.ErrorF("db service node error:%v", err)
		return nil, pb.ErrCode_SystemErr
	}
	us := &user.GameUser{
		User: user.User{
			ID: uid,
		},
		UserResources: user.UserResources{
			UID: uid,
		},
	}
	rpcMsg := ipb.MakeRpcMsg[user.GameUser](rpcName.GetGameUser, uid, us)
	rspMsg, err := s.Call(service.RpcTopicEx(node.Name), timeout, rpcMsg)
	if err != nil {
		log.ErrorF(s.Log("call rpc:%v err:%s ", rpcMsg.RpcMsgId, err.Error()))
		return nil, pb.ErrCode_SystemErr
	}
	_ = json.Unmarshal(rspMsg.Msg, us)
	return us, pb.ErrCode_OK
}

// 匹配房间
func (s *MatchService) onMatchRoom(iMsg *ipb.InternalMsg, req *pb.C2SMatchRoom) {
	ksync.GoSafe(func() {
		// color game无需进入匹配队列
		switch pb.ServiceTypeId(req.PlayType) {
		case pb.ServiceTypeId_STI_ColorGame:
			s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_S2CMatchRoomId), req)
			return
		}

		var us *user.GameUser
		rsp := &pb.S2CMatchRoom{}
		us, rsp.Code = s.getGameUser(iMsg.UserId)
		if rsp.Code != pb.ErrCode_OK {
			s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_S2CMatchRoomId), rsp)
			return
		}
		vipLv, vipExp := utils.VipLevel(us.VipExp)
		rsp.User = &pb.GameUser{
			UserId:      us.ID,
			Nickname:    us.Nickname,
			Avatar:      us.AvatarUrl,
			AvatarFrame: us.AvatarFrame,
			VipLevel:    vipLv,
			VipExp:      vipExp,
			Seat:        0,
			Gold:        us.Gold,
		}
		rsp.PlayType = req.PlayType
		rsp.RoomType = req.RoomType
		switch pb.ServiceTypeId(rsp.PlayType) {
		case pb.ServiceTypeId_STI_ColorGame:
			rsp.ColorInfo = &pb.S2CMatchRoom_ColorInfo{}
		}

		s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_S2CMatchRoomId), rsp)
	}, nil)
}
